import {Component, Input, OnInit} from '@angular/core';
import {Observation, ObservationType} from '@nspop/gm-web-facade-api';
import {DataCardWrapper} from '../../../model-wrappers/dataCard.wrapper';
import { ObservationFormType } from '../../../measurements/measurements-modal/measurements-modal.component';

@Component({
  selector: 'app-mother-related-measurements',
  templateUrl: './mother-related-measurements.component.html',
  styleUrls: ['./mother-related-measurements.component.scss']
})
export class MotherRelatedMeasurementsComponent implements OnInit {
  public ObservationType = ObservationType; // Exposed here so we can use actual enum in the html template

  @Input() public dataCard: DataCardWrapper | undefined;
  @Input() public observations: Observation[] | undefined;
  @Input() public observationType: ObservationFormType | undefined;

  constructor() {
  }

  ngOnInit(): void {
  }

  public getMeasurement(type: ObservationType): Observation | undefined {
    return this.observations
      ?.filter(x => x.type === type)
      .sort((a, b) => b.dateOfMeasurement.getTime() - a.dateOfMeasurement.getTime())?.[0];
  }

  public getMultipleMeasurementValues(type: ObservationType, includeUnit: boolean = false): string {
    const measurements = this.getMeasurement(type)?.values || [];
    if (measurements.length === 0) {
      return '';
    }

    const measurementStrings = measurements.map(measurement => {
      let stringBuilder = `${measurement?.value || ''}`;
      if (includeUnit) {
        stringBuilder += ` ${measurement?.unit}`;
      }

      return stringBuilder;
    });

    return measurementStrings.join(', ');
  }

  public getSingleMeasurementValue(type: ObservationType): string {
    const measurement = this.getMeasurement(type)?.values?.[0];
    if (!measurement) {
      return '';
    }
    return `${measurement.value || ''} ${measurement.unit || ''}`;
  }

  public getBloodPressure(): string {
    const observation = this.getMeasurement(this.ObservationType.BloodPressure);
    if (!observation) {
      return '';
    }
    return `${observation.values[0]?.value || ''} / ${observation.values[1]?.value || ''} ${observation.values[0]?.unit || ''}`;
  }
}
