import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';

import {PatientInformationComponent} from './patient-information.component';
import {of} from 'rxjs';
import {Component, Directive, forwardRef, Input} from '@angular/core';
import {ControlValueAccessor, FormsModule, NG_VALUE_ACCESSOR} from '@angular/forms';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {CardComponent} from '../card/card.component';
import {NamePipe} from './name.pipe';
import {OptionalDatePipe} from './optional-date.pipe';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {MatMenuModule} from '@angular/material/menu';
import {MatIconModule} from '@angular/material/icon';
import {MaskableInputComponent} from '../maskable-input/maskable-input.component';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {DataCardService} from '../api/data-card.service';
import {createDataCardServiceSpy} from '../api/data-card.service.spec';
import {getPropertyAsSpy} from '../../utils/spy';
import SpyObj = jasmine.SpyObj;
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {GeneralInformationBarComponent} from '../general-information-bar/general-information-bar.component';
import {CarePlanService} from '../api/care-plan.service';
import {SorService} from '../services/sor.service';
import {EncounterService} from '../services/encounter.service';
import {SemiFixedHeaderComponent} from '../semi-fixed-header/semi-fixed-header.component';
import {PatientInformationInputsComponent} from './patient-information-inputs/patient-information-inputs.component';

@Component({
  selector: 'app-radio-group',
  template: '<ng-content></ng-content>'
})
class MockGroupWrapperComponent {
}

const APP_MOCK_PATIENT_INPUTS_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MockPatientInputsComponent),
  multi: true
};
@Component({
  selector: 'app-patient-information-inputs',
  template: '',
  providers: [APP_MOCK_PATIENT_INPUTS_ACCESSOR]
})
class MockPatientInputsComponent implements ControlValueAccessor{
  registerOnChange(fn: any): void {
  }
  registerOnTouched(fn: any): void {
  }
  writeValue(obj: any): void {
  }
}

// tslint:disable-next-line:directive-selector
@Directive({selector: 'mask'})
class MaskMockDirective {
}

describe('PatientInformationComponent', () => {
  let component: PatientInformationComponent;
  let fixture: ComponentFixture<PatientInformationComponent>;

  let dataCardServiceStub: SpyObj<DataCardService>;

  beforeEach(async () => {
    dataCardServiceStub = createDataCardServiceSpy();

    dataCardServiceStub.update.and.returnValue(of({} as any));
    dataCardServiceStub.refresh.and.returnValue(of({} as any).toPromise());
    getPropertyAsSpy(dataCardServiceStub, 'isDataCardClosed').and.returnValue(of(false));
    getPropertyAsSpy(dataCardServiceStub, 'selectedDataCard$').and.returnValue(of() as any);

    const sorServiceStub = {
      sorLookupOrganizations$: of([])
    } as any;

    await TestBed.configureTestingModule({
      declarations: [
        PatientInformationComponent,
        MockGroupWrapperComponent,
        GeneralInformationBarComponent,
        MockPatientInputsComponent,
        SemiFixedHeaderComponent,
        MaskMockDirective,
        NamePipe
      ],
      imports: [
        NoopAnimationsModule,
        FormsModule,
        AngularMaterialModule
      ],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: SorService, useValue: sorServiceStub},
        {provide : EncounterService, useValue: of({})}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PatientInformationComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('disables save button while saving', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.isSubmitting).toBeFalse();
    component.save();
    expect(component.isSubmitting).toBeTrue();
    flush();
    expect(component.isSubmitting).toBeFalse();
  }));


  it('handles that active datacard could be undefined', fakeAsync(() => {
    fixture.detectChanges();
    expect(component.dataCard).toBeUndefined();
  }));
});
