import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {RoleSelectorComponent} from './role-selector.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {UserAuthorizationType, UserContextService} from '../services/user-context.service';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {of} from 'rxjs';
import {SorItem, SorService} from '../services/sor.service';
import {Injectable} from '@angular/core';
import {DialogService} from '../modals/dialog/dialog.service';
import {CarePlan} from '@nspop/gm-web-facade-api';

describe('RoleSelectorComponents\'s', () => {
  describe('lookup functionality', () => {
    let component: RoleSelectorComponent;
    let fixture: ComponentFixture<RoleSelectorComponent>;
    let sorService: SorService;
    let userContextService: UserContextService;
    const dialogServiceStub = {
      close: () => {}
    } as any;
    const userContextServiceStub = {
      eligibleRoles$: of([{
        authorizationCode: 'acode1234',
        educationCode: 'eduCode',
        educationType: 'eduType'
      }]),
      selectOrganizationSor: () => {},
      selectRole: () => {}
    } as any;

    beforeEach(async () => {
      await TestBed.configureTestingModule({
        declarations: [RoleSelectorComponent],
        imports: [AngularMaterialModule, NoopAnimationsModule, HttpClientTestingModule],
        providers: [
          {provide: UserContextService, useValue: userContextServiceStub},
          {provide: DialogService, useValue: dialogServiceStub},
        ]
      })
        .compileComponents();
    });

    beforeEach(() => {
      fixture = TestBed.createComponent(RoleSelectorComponent);
      component = fixture.componentInstance;
      sorService = TestBed.inject(SorService);
      userContextService = TestBed.inject(UserContextService);
      fixture.detectChanges();
    });

    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should preselect a role if only one is available', () => {
      expect(component.selectedRole?.educationCode).toBe('eduCode');
    });

    it('should not close the dialog when continue is triggered but either selectedRole or selectedOrganization is undefined', () => {
      spyOn(dialogServiceStub, 'close').and.callThrough();
      component.continue();
      expect(dialogServiceStub.close).not.toHaveBeenCalled();

      component.selectedOrganisation = {institutionOwnerSorId: '123', sorId: '1', name: 'Hello Org'};
      component.selectedRole = {educationType: 'abc', educationCode: '1234', authorizationCode: 'code1337' };
      component.continue();
      expect(dialogServiceStub.close).toHaveBeenCalled();
    });

    it('should ensure that both role and organization is set via the userContextService upon continue', () => {
      spyOn(userContextServiceStub, 'selectOrganizationSor').and.callThrough();
      spyOn(userContextServiceStub, 'selectRole').and.callThrough();
      component.selectedOrganisation = {institutionOwnerSorId: '123', sorId: '1', name: 'Hello Org'};
      component.selectedRole = {educationType: 'abc', educationCode: '1234', authorizationCode: 'code1337' };
      component.continue();
      expect(userContextServiceStub.selectOrganizationSor).toHaveBeenCalled();
      expect(userContextServiceStub.selectRole).toHaveBeenCalled();
    });
  });
});
