import { Directive, Input } from '@angular/core';
import { AbstractControl, NG_VALIDATORS, ValidationErrors, Validator } from '@angular/forms';

@Directive({
  selector: '[appAllowOnlyNumbers], [maxDecimals], [maxDigits], [isInteger]',
  providers: [{provide: NG_VALIDATORS, useExisting: AllowOnlyNumbersDirective, multi: true}]
})
export class AllowOnlyNumbersDirective implements Validator {

  @Input() maxDecimals = NaN;
  @Input() maxDigits = NaN;
  @Input() isInteger = false;

  constructor() {
  }

  validate(c: AbstractControl): ValidationErrors | null {
    const value: string | null = c.value;

    if (value == null) {
      return null;
    }


    if (value.length > 1 && value.startsWith('0') && !value.startsWith(`0.`)) {
      return { appOnlyNumbersDirective: 'No leading zeros!' };
    }
    const endsWith = (originalString: string, search: string, strLength?: number) => {
      if (strLength === undefined || strLength > originalString.length) {
        strLength = originalString.length;
      }
      return originalString.substring(strLength - search.length, strLength) === search;
    };
    if (value.length > 1 && endsWith(value, '.')) {
      return { appOnlyNumbersDirective: 'No ending decimal separators!' };
    }

    const numberRegExp = this.getNumberRegExp(this.maxDigits, this.maxDecimals);
    if (!numberRegExp.test(value)) {
      return { appOnlyNumbersDirective: 'Value is not a valid or accepted number!' };
    }

    return null;
  }

  private getNumberRegExp(maxDigits?: number, maxDecimals?: number): RegExp {
    const digitsAmountString = !maxDigits ? '*' : `{0,${maxDigits}}`;

    if (this.isInteger || this.maxDecimals === 0) {
      return new RegExp(`^\\d${digitsAmountString}$`);
    }

    const decimalsAmountString =  !maxDecimals ? '*' : `{0,${maxDecimals}}`;
    return new RegExp(`^\\d${digitsAmountString}(\\.\\d${decimalsAmountString})?$`);
  }
}
